----------------------------------------------------------------------------------
--
-- LGP-30 drum computer simulation
-- Simple USB Transceiver, modeled after Philips ISP 1105
--
-- jm 30.7.2017
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity usb_xcvr is
	Port ( 

		-- external USB bus
		usb_dp : inout  STD_LOGIC;				-- Data+
		usb_dn : inout  STD_LOGIC;				-- Data-
			  
		-- internal I/O and control
		rxd, rxdp, rxdn  : out  std_logic;
		txdp, txdn, txoe : in std_logic;
		
		mode : in STD_LOGIC := '1'				-- high for differential signaling, low for txdn = FSEO
	);
end usb_xcvr;

architecture Behavioral of usb_xcvr is

begin

	-- transmit
	process (txoe, txdp, txdn, mode) is
	begin
		if txoe = '1' then		-- not enabled
			usb_dp <= 'Z';
			usb_dn <= 'Z';
		else
			if mode = '1' then		-- differential drive
				usb_dp <= txdp;
				usb_dn <= txdn;
			else							-- single ended
				if txdn = '1' then	-- FSE0 asserted
					usb_dp <= '0';
					usb_dn <= '0';
				else
					usb_dp <= txdp;
					usb_dn <= txdn;
				end if;
			end if;
		end if;
	end process;
	
	-- receive
	process (usb_dp, usb_dn) is
	begin
		rxdp <= usb_dp;
		rxdn <= usb_dn;
		if (usb_dp = '0' and usb_dn = '0') then	-- SE0
			-- rxd stays unchanged
		else
			rxd <= usb_dp;
		end if;
	end process;
		
end Behavioral;

